function ccrs = feature_hallucinate(Xi, Xo, L, tol, splits, weights, sigma, T, B, conf_measure)
% usage: ccrs = feature_hallucinate(Xi, Xo, L, tol, splits, weights, sigma, T, B, conf_measure)
%  Runs feature hallucination approach over specified train/test splits.
%
% Input:
%  * Xi, Xo: input and output distance matrices
%  * L: 1 x N vector of integer labels
%  * tol: percent model variance maintained by kpca space (default -0.95, see kpca.m)
%  * splits: 1 x P vector of split data structures
%  * weights: 1 x V vector of view weights
%  * sigma: noise variance (default 1e-2)
%  * T: number of bootstrap iterations (default 0)
%  * B: bootstrap number of samples to add per class
%  * conf_measure: either 'variance' or 'margin' (default 'margin')
%
% Output:
%  * ccrs: P x 1 vector of split ccrs 
%

% define covariance function
covfun = 'covexpsum';
  
% get tolerance
if exist('tol')~=1 || isempty(tol)
  tol = -0.95;
end
  
% get sigma
if exist('sigma')~=1 || isempty(sigma)
  sigma = 1e-2;
end

% get number of boostrap iterations
if exist('T')~=1 || isempty(T)
  T = 0;
else
  if exist('B')~=1 || isempty(B)
    error('B not specified');
  end
end

% get confidence measure
if exist('conf_measure')~=1 || isempty(conf_measure)
  conf_measure = 'margin';
end

% properties
N = length(L);
nSplits = length(splits);

% initialize output structures
ccrs = zeros(nSplits,1);

% set dimension
d = tol;

% iterate over splits
for sId = 1:nSplits  
  % method counter init
  method = 1;
  
  % prompt user
  fprintf('Split %d/%d...\n', sId, nSplits);

  % define split
  train = splits(sId).train;
  test = splits(sId).test;
  
  % properties
  n = length(train);
  m = length(test);

  % compute kpca on test data
  tXo = Xo(test,test);
  bwd = 1./mean(tXo(:));
  txo = kpca(exp(-bwd*tXo),tol);
  
  % infer output modality over training set
  pxo = zeros(size(txo,1),N);
  pxo(:,test) = txo;

  bwd = 1./mean(Xi(:));
  Ki = exp(-bwd*Xi);
  pxo(:,train) = (Ki(train,test)*inv(Ki(test,test) + sigma*eye(m))*(txo'))';

  % evaluate inferred model
  pXo = sq_dist(pxo);
  logtheta_input = log(1./mean(Xi(:)));  
  logtheta_output = log(1./mean(pXo(:)));
  logtheta = [log(weights) logtheta_input logtheta_output];
  ccrs(sId) = gpeval(logtheta,covfun,{Xi,pXo},L,train,test,sigma);
  
  % run bootstrapping
  if(T>0)
    bsA = gpbootstrap({Xi,pXo},covfun,logtheta,sigma,L,train,test,B,T,conf_measure);
    ccrs(sId) = bsA(end);
  end
  
end

% prompt user
fprintf('done.\n\n');
