function [ccrs,pL,S] = gpbootstrap(X, covfun, logtheta, sigma, L, S, U, N, T, confmeasure)
% usage: [ccrs,pL,S] = gpbootstrap(X, covfun, logtheta, sigma, L, S, U, N, T, confmeasure)
%  GP Bootstrap.
%
% Input:
%  * X: input observations (format depends on covfun)
%  * covfun: handle to GP covariance function
%  * logtheta: covariance function hyperparameters (log domain)
%  * sigma: GP noise variance
%  * L: dataset ground truth labels
%  * S: vector of labeled point indexes
%  * U: vector of unlabeled point indexes
%  * N: number of samples to add per class
%  * T: number of rounds
%  * confmeasure: employed confidence measure ('variance' or 'margin')
%     (default 'variance')
%
% Output:
%  * ccrs: 1 x T vector of classification accuracies over each round
%  * pL: predicted class labels from the last round
%  * S: vector of indexes of final expanded labelled set
%  

if exist('confmeasure')~=1 || isempty(confmeasure)
  confmeasure = 'variance';
end

C = unique(L);
nC = length(C);

ccrs = zeros(1,T);
pL = L;

Uorig = U;

for t = 1:T
  % classify and assign to pL
  [ccrs(t),cL,nll,f,vars] = gpeval(logtheta,covfun,X,pL,S,U,sigma);
  pL(U) = cL;
  ccrs(t) = ccr(pL(Uorig),L(Uorig));
  
  % prompt user
  fprintf('Iteration %d/%d: |S|=%d, |U|=%d, CCR=%5.2f%%\n', ...
          t, T, length(S), length(U), ccrs(t)*100);  
  
  % expand S, shrink U
  for c = 1:nC
    ids = find(cL==C(c));
    
    switch confmeasure
     case 'variance'
      if nC>2
        confs = vars(ids,c);
      else
        confs = vars(ids);
      end
      direction = 'ascend';
     case 'margin'
      if nC>2
        confs = f(ids,c);
      else
        confs = (3-2*c).*f(ids);
      end
      direction = 'descend';
     otherwise
      error('''%s'' is not a valid confidence measure', confmeasure);
    end

    [confs, sIds] = sort(confs, direction);
    ids = ids(sIds);

    nsamps = min(length(ids),N);
    ids = ids(1:nsamps);
    S = [S; U(ids)];
    U(ids) = [];
    cL(ids) = [];
    
    if isempty(U)
      ccrs = ccrs(1:t);
      return;
    end
  end
  
end

