function [x,evals,evecs] = kpca(K, d)
% usage: [x,evals,evecs] = kpca(K, d)
%  Performs kernel PCA.
%
% Input:
%  * K: N x N feature kernel matrix
%  * d: dimensionality of latent space (at most N, default N)
%       (if d < 0 its treated as a tolerance, i.e., -d percent model
%        variance is retained)
%
% Output:
%  * x: d x N latent space features
%  * evals: 1 x N eigenvalue vector
%  * evecs: N x N eigenvector matrix
%

% get number of samples
[N,M] = size(K);
if N~=M
  error('kernel matrix is not square');
end

% get d
if exist('d')~=1 || isempty(d)
  d = N;
end

% check that d is in bounds
if d<-1 || d>N || (d>=0 && d<1)
  error('d must be between -1 and 0 or 1 and N');
end

% center in feature space
unit = ones(N,N)/N; 
K_n = K - unit*K - K*unit + unit*K*unit; 

% compute principle components
[evecs,evals] = eig(K_n);
evals = real(diag(evals)); 

% normalize eigenvectors to unit norm
for ii = 1:N
 evecs(:,ii) = evecs(:,ii)/(sqrt(evals(ii))); 
end 

% sort eigenvalues in descending order
[evals,ids] = sort(evals,'descend');
evecs = evecs(:,ids);

% compute d if tolerance is specified
if d<0
  tol = -d;
  pvar = cumsum(evals)/sum(evals);
  d = find(pvar>=tol);
  if isempty(d)
    d = N;
  end
  d = d(1);
  fprintf('Retaining %d dimensions\n', d);
end

% compute features
x = (K_n * evecs(:,1:d))';
